/*
 * Classified Tydeman Consulting Proprietary
 * Copyright (C) 1994, 2000 Tydeman Consulting.  All rights reserved.
 *
 *  Fred Tydeman        tydeman@tybor.com
 *
 *  This should be compiler vendor provided.
 *
 *  If missing, here is one which can be used with the sample tests for
 *  x86/x87 systems and another that might be useable with Sun SPARCs.
 *
 * MODIFICATION HISTORY:
 *
 * 2000/03/01 - Make classification explicit in each file.
 * 2000/03/01 - Remove confidential so can post to public ftp site.
 *
 ***********************************************************************/

#include "mydefs.h"		/* FENV_H */
#include FENV_H			/* fegetround(), FE_* */

#if (SAMPLE_MISSING == 1)	/* Intel x86/x87 w/ _control87() */

#include <float.h>		/* _control87() */

int
fegetround(void)
{
    unsigned int     ui;
    int              ret;

    ui = _control87(0u, 0u);	/* get current '87 control word */
    ui &= 0x0c00u;		/* extract 2 bits: rounding control */
    ui = ui >> 10u;
    switch (ui) {
    case 0u:
	ret = FE_TONEAREST;
	break;
    case 1u:
	ret = FE_DOWNWARD;
	break;
    case 2u:
	ret = FE_UPWARD;
	break;
    case 3u:
	ret = FE_TOWARDZERO;
	break;
    default:
	ret = -1;		/* can't happen */
    }				/* switch */
    return ret;
}

#elif (SAMPLE_MISSING == 2)	/* This may work for Sun SPARC systems */

int
ieee_flags(const char *, const char *, const char *, char **);

#include <string.h>		/* strcmp() */

int
fegetround(void)
{
    const char      *out;
    int              ret = -1;	/* in case no match */

    (void) ieee_flags("get", "direction", "", &out);
    if (strcmp(out, "nearest") == 0)
	ret = FE_TONEAREST;
    if (strcmp(out, "tozero") == 0)
	ret = FE_TOWARDZERO;
    if (strcmp(out, "positive") == 0)
	ret = FE_UPWARD;
    if (strcmp(out, "negative") == 0)
	ret = FE_DOWNWARD;
    return ret;
}

#else /* Dummy */

#include <float.h>		/* FLT_ROUNDS */

int
fegetround(void)
{
    return (int) FLT_ROUNDS;
}

#endif
