/*
 * Classified Tydeman Consulting Proprietary
 * Copyright (C) 1994, 2000 Tydeman Consulting.  All rights reserved.
 *
 *  Fred Tydeman        tydeman@tybor.com
 *
 *  This should be compiler vendor provided.
 *
 *  If missing, here is a sample which can be used with the sample tests
 *  on x86/x87 systems and perhaps Sun SPARCs.
 *
 * MODIFICATION HISTORY:
 *
 * 2000/03/01 - Make classification explicit in each file.
 * 2000/03/01 - Remove confidential so can post to public ftp site.
 *
 ***********************************************************************/

#include "mydefs.h"		/* FENV_H */
#include FENV_H			/* fetestexcept(), FE_* */

#if (SAMPLE_MISSING == 1)	/* Intel x86/x87 w/ _status87() */

#include <float.h>		/* _status87() */

int
fetestexcept(int s)
{				/* tests status flags in FPU status word */
    signed int       si;
    unsigned int     ui2;

    ui2 = _status87();		/* get FPU status flags */
    si = s & FE_ALL_EXCEPT;	/* only check sticky flags */

    return (si & (signed int) ui2);	/* bit-wise AND of bits set and bits
					   wanted */
}

#elif (SAMPLE_MISSING == 2)	/* This may work for Sun SPARC systems */

int
ieee_flags(const char *, const char *, const char *, char **);

#include <string.h>		/* strcmp() */

int
fetestexcept(int s)
{				/* tests status flags in FPU status word */

    const char      *out;
    int              si = 0;

    if (s & FE_INVALID) {
	(void) ieee_flags("get", "exception", "invalid", &out);
	if (strcmp(out, "invalid") == 0)
	    si |= FE_INVALID;
    }
    if (s & FE_DIVBYZERO) {
	(void) ieee_flags("get", "exception", "division", &out);
	if (strcmp(out, "division") == 0)
	    si |= FE_DIVBYZERO;
    }
    if (s & FE_OVERFLOW) {
	(void) ieee_flags("get", "exception", "overflow", &out);
	if (strcmp(out, "overflow") == 0)
	    si |= FE_OVERFLOW;
    }
    if (s & FE_UNDERFLOW) {
	(void) ieee_flags("get", "exception", "underflow", &out);
	if (strcmp(out, "underflow") == 0)
	    si |= FE_UNDERFLOW;
    }
    if (s & FE_INEXACT) {
	(void) ieee_flags("get", "exception", "inexact", &out);
	if (strcmp(out, "inexact") == 0)
	    si |= FE_INEXACT;
    }
    return si;
}

#else /* Dummy */

int
fetestexcept(int s)
{
    if( s ){		/* use the var to keep lint happy */
    }
    return 0;
}

#endif
