/*
 *  Classified Tydeman Consulting Proprietary
 *  Copyright (C) 2006, 2013 Tydeman Consulting.  All rights reserved.
 *
 *  Fred J. Tydeman	tydeman@tybor.com
 *
 * - -- --- ---- ----- -------------------------------------------------
 *
 *    This code is copyrighted.  Under the copyright laws, this code may
 *	   not be copied, in whole or part, without prior written
 *	   consent of Tydeman Consulting.
 *
 *    RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the
 *	   government is subject to restrictions as set forth in
 *	   subparagraph (c)(1)(ii) of the Rights in Technical Data and
 *	   Computer Software clause at DFARS 252.227-7013 and
 *	   FAR 52.227-19.
 *
 * - -- --- ---- ----- -------------------------------------------------
 * $Revision: 2.1 $
 ***********************************************************************
 *  FILE: g1errno.h
 *
 *  Assertion being tested: errno and FP exception flags are consistent,
 *  and match what math_errhandling indicates.
 *
 *  NOTE:  Replace 'q' in '_q' with name of function, eg, sin, if you
 *  need to define a symbol for your *.bug file.
 *
 *  Which flags are tested is controled by:
 *	TEST_OPT_FLAGS == OR in optional flags to required flags.
 *	IMPL_NO_TEST_CRIT_FLAGS == Do NOT use critical actual flags.
 *	IMPL_NO_TEST_ACTUAL_FLAGS == Do NOT use actual flags.
 *
 *  MODIFICATION HISTORY:
 *
 * 2006/03/29 - Create.
 * 2006/06/21 - Make into function.
 * 2006/06/21 - Update DEBUG section.
 * 2006/06/21 - Indent #if's.
 * 2006/08/26 - Add print_val_q().
 * 2008/07/14 - Shorten marco symbols to less than 32 chars.
 * 2008/08/25 - Add IMPL_SPURIOUS_ERRNO.
 * 2011/01/06 - FIX_*_q -> LMS_*_q; IMPL_*_q -> LCL_*_q.
 * 2013/05/12 - Fix spelling of weird.
 *
 ***********************************************************************/

#ifdef DEBUG	/* local override of FIXes to force errors to show */
  #ifdef DEBUG2	/* Kills some compilers */
    #undef  FIX_SYM_math_errhandling
  #endif
#endif
#ifdef DEBUG	/* local override of FIXes to force errors to show */
  #ifdef DEBUG3
    #undef  IMPL_NO_TEST_ACTUAL_FLAGS
    #undef  IMPL_NO_TEST_CRIT_FLAGS
    #undef  IMPL_SPURIOUS_ERRNO
    #undef  LCL_UF_WEIRD_q
  #endif
  #undef  LMS_FLAG_q
  #define TEST_OPT_FLAGS
#endif

static int chk_errno_flags( const int errno1,
			    const int flags1,
			    const vector vd[],
			    const unsigned int vectNum
			  ){
  unsigned long int oldNumFail = NumFail;
  int flags_matter = 0;
  int rc = 0;		/* assume all will be OK */

  if( vd ){
    flags_matter  = vd[vectNum].flags_req;	/* Flags we expect */
#ifdef TEST_OPT_FLAGS
    flags_matter |= vd[vectNum].flags_opt;
#endif
#if (!defined IMPL_NO_TEST_CRIT_FLAGS)
    if( flags1 & (FE_IEEE_EXCEPT & (~FE_INEXACT)) ){
      flags_matter = flags1 ;		/* Use critical flags we got */
    }
#endif
  }
#if (!defined IMPL_NO_TEST_ACTUAL_FLAGS) \
 && (!defined FIX_SYM_math_errhandling) \
 && (defined MATH_ERREXCEPT)
  if( math_errhandling & MATH_ERREXCEPT ){
    flags_matter = flags1 ;		/* Use all flags we got */
  }
#endif

 flags_matter &= FE_IEEE_EXCEPT;	/* remove non-IEEE flags */

#if (!defined LMS_FLAG_q)
 /*
  * See if more than one flag raised (including inexact).
  */
 if( flags_matter & FE_INVALID ){
   ck( 0 == (flags_matter & ~FE_INVALID) );
   if( ckFailed ){
     (void)printf(" flags_matter = %#x\n", (unsigned int)flags_matter);
   }
 }
 if( flags_matter & FE_DIVBYZERO ){
   ck( 0 == (flags_matter & ~FE_DIVBYZERO) );
 }
#endif

 flags_matter &= ~FE_INEXACT;	/* remove inexact */

#if (!defined LMS_FLAG_q)
 /*
  * See if more than one flag raised (ignoring inexact).
  */
 if( flags_matter & FE_OVERFLOW ){
   ck( 0 == (flags_matter & ~FE_OVERFLOW) );
 }
 if( flags_matter & FE_UNDERFLOW ){
   ck( 0 == (flags_matter & ~FE_UNDERFLOW) );
 }
#endif

#if (defined MATH_ERRNO) && (!defined FIX_SYM_math_errhandling)
 if( 0 == (math_errhandling & MATH_ERRNO) ){
  #if (defined IMPL_SPURIOUS_ERRNO)
   if( 0 != errno1 ){	/* see if "spurious" errno matches flags */
     if(FE_INVALID   & flags_matter){
       ck( (EDOM   == errno1) == (FE_INVALID   == flags_matter) );
     }
     if(FE_DIVBYZERO & flags_matter){
       ck( (ePOLE  == errno1) == (FE_DIVBYZERO == flags_matter) );
     }
     if(FE_OVERFLOW  & flags_matter){
       ck( (ERANGE == errno1) == (FE_OVERFLOW  == flags_matter) );
     }
    #ifndef LCL_UF_WEIRD_q
     if(FE_UNDERFLOW & flags_matter){
       ck( (ERANGE == errno1) == (FE_UNDERFLOW == flags_matter) );
     }
    #endif
   }else
  #endif
   {
     ck( (0 == errno1) && "errno not always 0" );
   }
 }else
#endif
 {
   if( flags_matter ){
#ifdef LCL_UF_WEIRD_q
     if(FE_UNDERFLOW != flags_matter)
#endif
       {
	 ck( errno1 && "errno not altered w/ required exceptions" );
       }
     if(FE_INVALID   & flags_matter){
       ck( (EDOM   == errno1) == (FE_INVALID   == flags_matter) );
     }
     if(FE_DIVBYZERO & flags_matter){
       ck( (ePOLE  == errno1) == (FE_DIVBYZERO == flags_matter) );
     }
     if(FE_OVERFLOW  & flags_matter){
       ck( (ERANGE == errno1) == (FE_OVERFLOW  == flags_matter) );
     }
#ifndef LCL_UF_WEIRD_q
     if(FE_UNDERFLOW & flags_matter){
       ck( (ERANGE == errno1) == (FE_UNDERFLOW == flags_matter) );
     }
#endif
   }else{
     ck( (0 == errno1) && "errno altered w/o required exception" );
   }
 }

 if( oldNumFail != NumFail ){
   rc = 1;	/* indicate that we had a problem */
   (void)printf(" Test vector %u: errno inconsistent w/ FP exception flags\n",
		vectNum);
#if 0	/* Removed; should be done by caller */

  #ifndef   PRINT_RND
    #define PRINT_RND /* no rounding direction to print */
  #endif

  #ifndef   PRINT_RES
    #define PRINT_RES PRINT_VAL("      res1", res1)
  #endif
   PRINT_RND;
   PRINT_ARGS;
   PRINT_RES;

#endif
   (void)printf(" errno1=%i, EDOM=%i, ERANGE=%i, ePOLE=%i\n",
		(int)errno1, (int)EDOM, (int)ERANGE, (int)ePOLE);
   (void)printf(" flags1=%#x; flags_matter=%#x",
		(unsigned int)flags1, (unsigned int)flags_matter);
   flags_matter |= flags1;
   if( flags_matter ){
     (void)printf(" =>");
   }
   if( FE_INVALID & flags_matter ){
     (void)printf(" FE_INVALID=%#x", (unsigned int)FE_INVALID);
   }
   if( FE_DIVBYZERO & flags_matter ){
     (void)printf(" FE_DIVBYZERO=%#x", (unsigned int)FE_DIVBYZERO);
   }
   if( FE_OVERFLOW & flags_matter ){
     (void)printf(" FE_OVERFLOW=%#x", (unsigned int)FE_OVERFLOW);
   }
   if( FE_UNDERFLOW & flags_matter ){
     (void)printf(" FE_UNDERFLOW=%#x", (unsigned int)FE_UNDERFLOW);
   }
   if( FE_INEXACT & flags_matter ){
     (void)printf(" FE_INEXACT=%#x", (unsigned int)FE_INEXACT);
   }
   (void)printf("\n");
#if (defined MATH_ERRNO)
   (void)printf(" MATH_ERRNO=%i", (int)MATH_ERRNO);
#endif
#if (defined MATH_ERREXCEPT)
   (void)printf(", MATH_ERREXCEPT=%i", (int)MATH_ERREXCEPT);
#endif
#if (!defined FIX_SYM_math_errhandling)
   (void)printf(", math_errhandling=%i", (int)math_errhandling);
#endif
#if (defined MATH_ERRNO) || (defined MATH_ERREXCEPT) || (!defined FIX_SYM_math_errhandling)
   (void)printf("\n");
#endif
 }

 return rc;
}/* chk_errno_flags */
